;; Programs for exploring edge classes with planar7.

(in-package '*lisp)

;; Tabulate class sizes:

(defun clsz ()
  (*set cntx (!! 0))
  (*when ntrve (*pset :add (!! 1) cntx edgc))
  (ppp cntx :end Ncl)
)

;; Report sizes of Up, Dn sets in current Subgraph

(defun udsz ()
  (*when ntrve
    (list (*when Vup (*sum (!! 1))) (*when Vdn (*sum (!! 1)))))
)

;; Try greedy calculation on current graph

(defun greed ()
  (start (- (* 0.5 Dt)) 0.0)
  (newa 0.95)
  (print (cycle)) (terpri)
  (udsz)
)

;; Elementary program to calculate edge classes using greedy
;; approach, uses cycle, no attempt to optimize.  Doesn't
;; install new classes

(*proclaim '(type (signed-pvar NC) newc))
(*defvar newc (!! 0))      ;; New class number

(*proclaim '(type Boolean-pvar newf))
(*defvar newf nil!!)       ;; New class determined

(*proclaim '(type fixnum Nwn))
(defvar Nwn 0)             ;; Next new class number = number of same

(defun grclass ()
  (*set newc (!! 0))  (*set newf nil!!)  (setq Nwn 0)
  ;; while not all edges assigned
  (do (ud) ((not (*or (and!! ntrve (not!! newf)))))
    ;; Initialize with assigned edges out of calculation
    (start (- (* 0.5 Dt)) 0.0)
    (*when newf (*set Uup (!! -1.0)) (*set Udn Uup))
    ;; One cycle for greedy calculation
    (newa 0.95)
    (print (cycle)) (terpri)
    (print (setq ud (udsz))) (terpri)
    (if (plusp (car ud))    ;; Up edges are a class
      (*when (and!! ntrve Vup) (*set newc (!! Nwn)) (*set newf t!!)
	       (incf Nwn)))
    (if (plusp (cadr ud))   ;; Dn edges are a class
      (*when (and!! ntrve Vdn) (*set newc (!! Nwn)) (*set newf t!!)
	                    (incf Nwn)))
  ) ;; end do
  (format T "~D new indedpendent edge classes" Nwn)
)

;; Install new edge classes as found, e.g., by grclass.  New classes
;; are defined by newc, Nwn.

(defun newcl ()
  (*when ntrve (*set edgc newc))
  (setq Ncl Nwn)
  ;; Update interactions for new classes
  (*with-vp-set Vpinter
    (*when intrp
      (*set clab (pref!! edgc rpab :vp-set *default-vp-set*))
      (*set clcd (pref!! edgc rpcd :vp-set *default-vp-set*))))
)

;; Tabulate interaction degrees per edge per class, also total
;; degree per edge.  Report maximums.  Assumes all interactions
;; present in vpinter.

(defun degrees ()
 (*let ((d (!! 0)))
       (declare (type (signed-pvar 24) d))
  (*set cnty (!! 0))    ;; To count maximum degree
  (*set cntx (!! 0))    ;; To count total degree
  (dotimes (kcl Ncl)
   (*set rnvz (!! 0))  (*set d (!! 0))
   (*with-vp-set vpinter
    (*when intrp
     (*when (=!! clab (!! kcl))
       (*pset :add (!! 1) rnvz rpcd :vp-set *default-vp-set*))
     (*when (=!! clcd (!! kcl))
       (*pset :add (!! 1) d rpab :vp-set *default-vp-set*)))
   ) ;; end *with-vp-set vpinter
   (*set d (+!! d rnvz))   ;; degree for class kcl
   (format t "class ~3D, max. degree ~3D" kcl (*max d))
   (if (oddp kcl) (terpri) (format t "    "))
   (*set cntx (+!! cntx d))
   (*set cnty (max!! cnty d))
  ) ;; end dotimes
  (format t "Maximum overall degree ~D~%" (*max cnty))
  (format t "Maximum total degree ~D" (*max cntx))
 )
)

